// dumpclusters.cpp
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html


#include <stdio.h>

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <qstring.h>

#include "ntfsutil.h"
#include "ntfsvolume.h"
#include "ntfsvolumeinfo.h"

void writeClusters(int fd, NT64 cluster, NT64 count, const char * fileName)
{
	// Make a volume attached to our file descriptor
	NTFSVolume v;
	v.attach(fd);

	// Get volume information
	const NTFSVolumeInfo * vi = v.getVolInfo();

	// Open a file by that name
	FILE * out = fopen(fileName, "w");
	if (out == NULL)
	{
		printf("Failed to open \"%s\" for writing. Reason: %s\n", fileName, strerror(errno));
		return;
	}

	if (v.copyData(vi->getPositionForLcn(cluster), vi->bytesPerCluster * count, fileno(out)))
	{
		printf("Wrote %u bytes to \"%s\".\n", (unsigned int)(vi->bytesPerCluster * count), fileName);
	}
	else
	{
		printf("Failed to retrieve data.\n");
	}

	// Close output file
	fclose(out);
}

// Entry point
int main(int argc, char ** argv)
{
	printf("SalvageNTFS Cluster Dump\nWritten by Will Glynn\n\n");

	if (argc != 5)
	{
		printf("Usage: %s <volume> <cluster number> <number of clusters> <output>\n", argv[0]);
		return 1;
	}

	int fd = open(argv[1], O_RDONLY | O_LARGEFILE);

	if (fd == -1)
	{
		printf("Unable to open %s for reading.\n", argv[1]);
		return 1;
	}

	NT64 offset = 0, count = 0;
	if (argc > 2)
		offset = toNum(argv[2]);
	if (argc > 3)
		count = toNum(argv[3]);

	writeClusters(fd, offset, count, argv[4]);

	if (close(fd) == -1)
	{
		printf("Unable to close the volume file.\n");
	}

	return 0;
}
