// ntfsattribute.h
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#ifndef NTFSATTRIBUTE_H
#define NTFSATTRIBUTE_H

#include "ntfs.h"
#include "ntfsstream.h"
#include "ntfsrunlist.h"
#include "ntfsattributedata.h"
#include <qcstring.h>

class NTFSAttribute
{
public:
	NTFSAttribute();
	~NTFSAttribute();
	void copy(NTFSAttribute * a);

	bool read(NTFSStream * s);
	QString toString();

	// Returns true if the attribute is resident or has been made resident; false otherwise
	bool deReference(NTFSVolume * v);
	// Makes s point to our data
	bool getData(NTFSVolume * v, NTFSStream * s);
	// Writing the attribute's contents directly to a file descriptor
	bool writeContentsToFile(NTFSVolume * v, int fd);

	// Common fields
	NT32 type;
	NT32 recordLength;
	NT8 nonResident;
	NT8 nameLength;
	NT16 nameOffset;
	NT16 flags;
	NT16 attributeId;

	// dataLength is read for resident attributes and computed for non-resident attributes
	NT32 dataLength;

	// Non-resident attribute only
	NT64 vcnStart;
	NT64 vcnStop;
	NT16 dataOffset;
	NT16 compressionUnitSize;
	NT64 sizeAlloc, sizeReal, sizeInitialized;
	NTFSRunList runList;

	QString name;

	// Data stream
	NTFSAttributeData * data;
};

#endif
