// ntfsattributes.h
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#ifndef NTFSATTRIBUTES_H
#define NTFSATTRIBUTES_H

#include "ntfs.h"
#include "ntfsstream.h"
#include "ntfsattribute.h"
#include "ntfsvolume.h"

#include <qstring.h>
#include <qptrlist.h>

class NTFSAttributes
{
public:
	NTFSAttributes();
	~NTFSAttributes();
	void zero();

	// Read attributes from a stream
	bool read(NTFSStream * s);

	// Returns a textual representation of the contained attributes
	QString toString();

	// Retrieves an attribute of given type (if any)
	NTFSAttribute * getAttribute(NT32 type);

	// Retrieves an attribute with the given type and ID (if any)
	NTFSAttribute * getAttributeById(NT32 type, NT16 id);

	// Returns all attributes of a given type
	const QPtrList<NTFSAttribute> getAttributesByType(NT32 type);

	// De-references any ATTRIBUTE_LISTs, bringing everything into
	bool deReference(NTFSVolume * v);

	// Helper functions
	QString getFileName();

protected:
	QPtrList<NTFSAttribute> m_list;

	// Combines runlists of non-resident attributes describing the same data stream into one
	bool combineRunLists(const NTFSVolumeInfo * vi);

};

#endif
