// ntfsfile.h
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#ifndef NTFSFILE_H
#define NTFSFILE_H

#include "ntfs.h"
#include "ntfsvolumeinfo.h"
#include "ntfsstream.h"
#include "ntfsattributes.h"

class NTFSRunList;
class NTFSFile
{
public:
	NTFSFile();
	~NTFSFile();

	// Returns a boolean indicating whether or not the data at s looks like a FILE
	// Not intended to be conclusive -- read() may still fail, but it's a quick check
	static bool identify(NTFSStream * s);

	// Read a FILE from a stream
	bool read(NTFSStream * s);

	// Returns whether or not this FILE is valid
	bool isValid();

	// Make a textual representation of the FILE
	QString toString();

	// Guts
	NT32 magic;
	NT16 updateSequenceOffset;
	NT16 updateSequenceSize;
	NT64 logSequence;
	NT16 recordSequence;
	NT16 hardLinkCount;
	NT16 attributeOffset;
	NT16 flags;
	NT32 recordSize;
	NT32 allocatedSize;
	NT64 baseFileRecord;
	NT16 nextAttributeId;

	NTFSAttributes a;

	// Our position on disk
	NT64 recordNum;

	// Helper stuff
	inline bool isDirectory() { return flags & 0x02; };
	inline bool isInUse() { return flags & 0x01; };

protected:
	// Our very own stream object
	NTFSStream * m_s;
};

#endif
