// ntfsrecovery.h
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#ifndef NTFSRECOVERY_H
#define NTFSRECOVERY_H

#include <stdio.h>

#include <qobject.h>
#include <qstring.h>

#include "ntfs.h"
#include "ntfsvolume.h"
#include "ntfsstream.h"
#include "ntfsindex.h"

class NTFSRecovery : public QObject
{
	Q_OBJECT

public:
    NTFSRecovery();
    ~NTFSRecovery();

	// Opens 'volume' (usually "/dev/hd__" block device) for recovery operations
	bool open(QString volume);
	// Closes any open volume
	void close();

	// Sets a target directory to recover stuff to
	bool setRecoveryArea(QString target);

	// Recovers a single file, ignoring directory structure
	bool recoverFile(NT64 record, QString path = "");
	// Recovers a directory tree, creating the directory, all child files, and all child directories
	bool recoverDirectory(NT64 record, QString path = "");

protected:
	// Volume-related
	QString volumeFileName;
	FILE * volumeFile;
	NTFSVolume v;

	// Recovery-related
	QString targetDir;
};

#endif
