// ntfsvolumeinfo.cpp
// Part of SalvageNTFS
// Copyright (C) Will Glynn
//////////////////////////////////////////////////////////////////////////////
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

#include "ntfsutil.h"
#include "ntfsvolumeinfo.h"
#include <string.h>

// Utility function; unneeded outside of NTFSVolumeInfo
unsigned int calculateShiftAmount(NT32 number)
{
	unsigned int shamt = 0;
	while (number != 1 && number > 0)
	{
		number = number >> 1;
		shamt ++;
	}
	return shamt;
}

// Default constructor
NTFSVolumeInfo::NTFSVolumeInfo()
{
	// Guess values
	bytesPerSector = 512;
	sectorsPerCluster = 8;
	serialNumber = 0;
	offsetToMft = 0;
	offsetToMftMirr = 0;
	bytesPerRecord = 1024;
	bytesPerIndex = 1024;
	sectorCount = 0;

	// Update shifts and other calculated values to reflect above defaults
	recalculate();
}

NTFSVolumeInfo::NTFSVolumeInfo(NTFSVolumeInfo &c)
{
	memcpy(this, &c, sizeof(NTFSVolumeInfo));
}

void NTFSVolumeInfo::recalculate()
{
	// Derive bytes per cluster
	bytesPerCluster = bytesPerSector * sectorsPerCluster;

	// Create shift values based on earlier numbers
	shiftLcnToPos = calculateShiftAmount(bytesPerCluster);
	shiftRecordToPos = calculateShiftAmount(bytesPerRecord);

	// Create masks based on earlier numbers
	// If record length is 0x400 (1024 bytes), then the mask should be 0x3FF (1023).
	// Thus, mask = length - 1.
	maskLcnOffset = bytesPerCluster - 1;
	maskRecordOffset = bytesPerRecord - 1;
}

QString NTFSVolumeInfo::toString()
{
	QString out;
	out.sprintf("Volume {\n\tBytes per sector: %i\n\tSectors per cluster: %i\n\tSerial number: %s\n\tBytes per record: %i\n\tBytes per index entry: %i\n\tNumber of sectors: %s\n}",
		bytesPerSector, sectorsPerCluster, toHex(serialNumber).latin1(),
		bytesPerRecord, bytesPerIndex, toHex(sectorCount).latin1());
	return out;
}

NT64 NTFSVolumeInfo::getRecordForReference(NT64 refnum) const
{
	// TODO: map MFT
	return 0x300000 + (refnum & 0xFFFFFFFFFFFF);
}

